pub struct RodioResampler<S: Source, T = f64>{ /* private fields */ }Expand description
Wrap a rodio::Source and resample it in realtime in your rodio pipeline. Requires the rodio feature.
When playing from a buffered audio source such as a file or a buffered stream, it is recommended to use config.with_rodio_fast_start(true), which will
avoid initial output delay by pulling samples from the upstream source to prime the resampler. For very-realtime sources such as microphones or similar,
do not enable fast-start.
Be aware that because RodioResampler resamples on the audio thread, your cpal buffer size should be at least 2048 to 4096. If you experience crackling, try increasing the cpal buffer size. Marginal buffer capacity first shows up as small glitches on seek.
Example:
let stream = rodio::DeviceSinkBuilder::open_default_sink()?;
let mixer = stream.mixer();
let tone = rodio::source::SignalGenerator::new(
NonZero::new(44_100 as u32).unwrap(),
400, // 400 Hz
rodio::source::Function::Sine,
)
.take_duration(Duration::from_secs(3.0));
let config = PRESET_FAST.with_channels(1).with_input_rate(44_100).with_output_rate(48_000);
let resampled_tone = RodioResampler::new(tone, config)?;
mixer.add(resampled_tone);
thread::sleep(Duration::from_secs(4));Implementations
Sourceimpl<S> RodioResampler<S, f64>where
S: Source,
impl<S> RodioResampler<S, f64>where
S: Source,
Sourcepub fn new(inner: S, config: Config) -> Result<Self, Error>
pub fn new(inner: S, config: Config) -> Result<Self, Error>
Create a new RodioResampler using f64 as the internal resampling type.
Config input sample rate and channel count can be a best-guess if you don’t know the exact values at the time of construction. If they are innacuate, a new span will be created when the actual values are known.
Sourceimpl<S> RodioResampler<S, f32>where
S: Source,
impl<S> RodioResampler<S, f32>where
S: Source,
Sourcepub fn new_f32(inner: S, config: Config) -> Result<Self, Error>
pub fn new_f32(inner: S, config: Config) -> Result<Self, Error>
Create a new RodioResampler using f32 as the internal resampling type.
Config input sample rate and channel count can be a best-guess if you don’t know the exact values at the time of construction. If they are innacuate, a new span will be created when the actual values are known.